module.exports = (function (state) {
  const R = require('ramda');
  const E = require('simple-encryptor')(state.get('masterPassword'));
  const T = require('egeria-tools');
  const fs = require('fs');
  const path = require('path');

  const frwoptions = 'utf8';
  const prefix = 'Identity manager |';
  state.set('prefix', prefix);

  const err = (message, origin, data) => T.err(prefix, message, origin, data);
  const logInfo = (message, origin, data) => T.log(state, 'info', message, origin, data);

  function fread (fname) {
    logInfo('Loading credentials from disk...');
    let action = 'While reading file ' + fname;
    return new Promise((resolve, reject) => {
      fs.readFile(fname, frwoptions, (e, data) => {
        if (!R.isNil(e)) {
          reject(err(action, e, {fname, frwoptions}));
        }
        else {
          resolve(data);
        }
      });
    });
  }

  function fwrite (fname, data) {
    logInfo('Saving credentials...');
    let action = 'While writing file ' + fname;
    return new Promise((resolve, reject) => {
      fs.writeFile(fname, data, frwoptions, (e, data) => {
        if (!R.isNil(e)) {
          reject(err(action, e, {fname, frwoptions}));
        }
        else {
          resolve(data);
        }
      });
    });
  }

  async function load (identity) {
    logInfo('Loading identity "' + identity + '"');
    let action = 'While loading identity ' + identity;
    let directory = state.get('directory');
    let filename = identity + '.dat';
    let encryptedCredentials;
    try {
      encryptedCredentials = await fread(path.resolve(directory, filename));
    }
    catch(e) {
      throw err(action, e, {directory, filename});
    }
    let decryptedCredentials = E.decrypt(encryptedCredentials);
    state.select('identities', identity).set(decryptedCredentials);
    return decryptedCredentials;
  }

  async function store (identity) {
    logInfo('Saving identity "' + identity + '"');
    let action = 'While storing identity ' + identity;
    let decryptedCredentials = state.select('identities', identity).get();
    let directory = state.get('directory');
    let filename = identity + '.dat';
    let encryptedCredentials = E.encrypt(decryptedCredentials);
    try {
      await fwrite(path.resolve(directory, filename), encryptedCredentials);
    }
    catch(e) {
      throw err(action, e, {directory, filename, decryptedCredentials, encryptedCredentials}, e);
    }
    return encryptedCredentials;
  }

  return {load, store};
});